/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Henrik Gemal.
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Henrik Gemal http://gemal.dk
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Put everything into a namespace or that we dont pollute
var LinkyContext = function() {
	// default prefs if we cant read the prefs
	this.pref_context_alllinks = true;
	this.pref_context_downloadlinks = false;
	this.pref_context_piclinks = true;
	this.pref_context_piclinksshow = true;
	this.pref_context_picshow = true;
	this.pref_context_selectedlinks = true;
	this.pref_context_selectedtextlinks = true;
	this.pref_context_validatelinks = false;
	this.pref_download_overwrite = 3;
	this.pref_imgexts = "jpg|jpeg|gif|png|bmp";
	this.pref_open_mailnews = false;
	this.pref_showopen = 2;
	this.pref_showselect = 0;
	this.pref_validateURL = "http://validator.w3.org/check?verbose=1&ss=1&uri=";
}

// get current selection as a string
LinkyContext.prototype.getSelectionString = function() {
	linkyShared.trace("getSelectionString");
	var focusedWindow = document.commandDispatcher.focusedWindow;
	var selection = focusedWindow.getSelection();;
	return String(selection);
}

// get current selection as a object
LinkyContext.prototype.getSelectionObject = function() {
	linkyShared.trace("getSelectionObject");
	var focusedWindow = document.commandDispatcher.focusedWindow;
	return focusedWindow.getSelection();
}

// can the link be opened or should we ignore it?
LinkyContext.prototype.isLinkOpenable = function(lnk, h) {
	linkyShared.trace("isLinkOpenable");
	if (!lnk) {
		return false;
	} else {
		if (linkyContext.isLinkType("javascript:", lnk)) {
			return false;
		} else if (linkyContext.isLinkType("mailto:", lnk) || linkyContext.isLinkType("news:", lnk) || linkyContext.isLinkType("snews:", lnk)) {
			if (this.pref_open_mailnews) {
				return true;
			} else {
				return false;
			}
		} else {
			if (h && !linkyContext.isLinkType("http:", lnk)) {
				return false;
			} else {
				return true;
			}
		}
	}
}

// can the link be saved or should we ignore it?
LinkyContext.prototype.isLinkSaveable = function(link) {
	linkyShared.trace("isLinkSaveable");
	return !(linkyContext.isLinkType("mailto:", link) || linkyContext.isLinkType("javascript:", link) || linkyContext.isLinkType("news:", link) || linkyContext.isLinkType("snews:", link));
}

// get the type from the link
LinkyContext.prototype.isLinkType = function(linktype, link) {
	linkyShared.trace("isLinkOpenable");
	try {
		var protocol = link.substr(0, linktype.length);
		return protocol.toLowerCase() == linktype;
	} catch(e) {
		linkyShared.trace("Unable to get protocol from " + link, 0, err);
		return false;
	}
}

// is the URL a reference to a image?
LinkyContext.prototype.isLinkImage = function(u) {
	linkyShared.trace("isLinkImage");
	var linky_imageurl = new RegExp("\." + this.pref_imgexts + "$", "i");
	if (!u) {
		linkyShared.trace("isLinkImage need URL");
		return false;
	} else {
		if (linky_imageurl.test(u)) {
			return true;
		} else {
			return false;
		}
	}
}

// get the referer from a document
LinkyContext.prototype.getReferer = function(d) {
	linkyShared.trace("getReferer");
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var loc;
	// do we have a location?
	if ("location" in d) {
		loc = d.location;
	} else {
		linkyShared.trace("No location in " + d, 0);
	}
	var ref = ioService.newURI(loc, null, null);
	linkyShared.trace("referer: " + ref.spec, 0);
	return ref;
}

// get a directory from a select a directory dialog
LinkyContext.prototype.getDirectory = function() {
	linkyShared.trace("getDirectory");
	var dir;
	var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance();
	if (filePicker != null) {
		filePicker = filePicker.QueryInterface(Components.interfaces.nsIFilePicker);
		if (filePicker != null) {
			try {
				var msg = document.getElementById("bundle_linky").getString("linky-download-select-directory");
				filePicker.init(window, msg, Components.interfaces.nsIFilePicker.modeGetFolder);
			} catch(err) {
				linkyShared.trace("Unable to get directory", 1, err);
			}
			filePicker.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
			if (filePicker.show() == Components.interfaces.nsIFilePicker.returnOK) {
				dir = filePicker.file;
			} else {
				linkyShared.trace("User cancalled out of director picker", 0);
			}
		} else {
			linkyShared.trace("Unable to open directory picker", 1);
		}
	} else {
		linkyShared.trace("Unable to init directory picker", 1);
	}
	if (dir && dir.path) {
		linkyShared.trace("selected directory: " + dir.path, 0);
		return dir.path;
	} else {
		linkyShared.trace("returning false from getDirectory", 0);
		return false;
	}
}

// Enable or disable the context menuitems
LinkyContext.prototype.contextState = function(p, ele) {
	linkyShared.trace("contextState");
	if (!p) {
		// yes all of them
		for (var i=0; i<ele.length; i++) {
			document.getElementById(ele[i]).setAttribute("hidden", true);
			linkyShared.trace("Hide " + ele[i]);
		}
		return false;
	} else {
		// perhaps some of them
		for (i=0; i<ele.length; i++) {
			if (((i == 1 || i == 3) && this.pref_showopen == 1) || ((i == 2 || i == 4) && this.pref_showopen == 0)) {
				document.getElementById(ele[i]).setAttribute("hidden", true);
				linkyShared.trace("UnHide " + ele[i]);
			} else {
				document.getElementById(ele[i]).setAttribute("hidden", false);
				linkyShared.trace("Hide " + ele[i]);
			}
		}
		return true;
	}
}

// Open a link either in a tab or a window
LinkyContext.prototype.openLink = function(lnk, typ, doc) {
	linkyShared.trace("openLink");
	if (!lnk) {
		linkyShared.trace("lnk in openLink is empty", 0);
		return;
	}
	// special case mail and news links
	if (linkyContext.isLinkType("mailto:", lnk) || linkyContext.isLinkType("news:", lnk) || linkyContext.isLinkType("snews:", lnk)) {
		try {
			window.loadURI(lnk);
		} catch(err) {
			linkyShared.trace("Unable to open mail or news link " + lnk, 1, err);
		}
	} else {
		if (typ == 1) {
			var browser;
			try {
				browser = getBrowser();
			} catch (ex if ex instanceof ReferenceError) {
				var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
				var browserWin = windowMediator.getMostRecentWindow("navigator:browser");
				if (!browserWin) {
					window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", lnk, null, linkyContext.getReferer(document.commandDispatcher.focusedWindow.document));
					return;
				} else {
					browser = browserWin.getBrowser();
				}
			}
			try {
				browser.addTab(lnk, linkyContext.getReferer(document.commandDispatcher.focusedWindow.document));
			} catch(err) {
				linkyShared.trace("Unable to open " + lnk, 1, err);
			}
		} else if (typ == 2) {
			try {
				document.commandDispatcher.focusedWindow.open(lnk);
			} catch(err) {
				linkyShared.trace("Unable to open " + lnk, 1, err);
			}
		} else if (typ == 3) {
			if (!doc) {
				linkyShared.trace("Document is undefined", 1);
			} else {
				var body = doc.getElementsByTagName("body")[0];
				if (!body) {
					body = doc.createElement("body");
					doc.appendChild(body);
				}
				var img = doc.createElement("img");
				img.src = lnk;
				img.setAttribute("alt", lnk);
				img.setAttribute("title", lnk);
				body.appendChild(img);
				var br = doc.createElement("br");
				body.appendChild(br);
			}
		} else {
			linkyShared.trace("openLink called with unknown typ:" + typ, 1);
		}
	}
}

// Generates a document in a new tab or window
// (Borrowd this method from Chris Pederick's Web Developer.)
LinkyContext.prototype.generateDocument = function(url, openInTab) {
	
    var generatedPage = null;
    var request = new XMLHttpRequest();
 
    if (openInTab) {
        getBrowser().selectedTab = getBrowser().addTab(url);
        generatedPage = window;
    } else {
        generatedPage = window.open(url);
    }

    // This must be done to make generated content render
    request.open("get", "about:blank", false);
    request.send(null);

    return generatedPage.content.document;
}

// enables and disables the context menu items
// p indicates where where the context menu was called:
//   0 or not indicated : Mozilla normal context menu
//   1 : From the Mozilla Firebird Go context menu
//   2 : From the Mozilla Mail context menu
LinkyContext.prototype.doContext = function(p) {
	linkyShared.trace("doContext");
	// context menuitems
	var elementTagsSelected;
	if (p == 1)
		elementTagsSelected = new Array('linky-selected-sep-go', 'linky-selected-tab-go', 'linky-selected-win-go', 'linky-validate-selected-tab-go', 'linky-validate-selected-win-go', 'linky-download-selected-go', 'linky-download-all-go');
	else if (p == 2)
		elementTagsSelected = new Array('linky-selected-sep-mail', 'linky-selected-tab-mail', 'linky-selected-win-mail', 'linky-validate-selected-tab-mail', 'linky-validate-selected-win-mail', 'linky-download-selected-mail', 'linky-download-all-mail');
	else
		elementTagsSelected = new Array('linky-selected-sep', 'linky-selected-tab', 'linky-selected-win', 'linky-validate-selected-tab', 'linky-validate-selected-win', 'linky-download-selected', 'linky-download-all');

	var elementTagsSelectedLink;
	if (p == 1)
		elementTagsSelectedLink = new Array('linky-selected-text-sep-go', 'linky-selected-text-tab-go', 'linky-selected-text-win-go');
	else if (p == 2)
		elementTagsSelectedLink = new Array('linky-selected-text-sep-mail', 'linky-selected-text-tab-mail', 'linky-selected-text-win-mail');
	else
		elementTagsSelectedLink = new Array('linky-selected-text-sep', 'linky-selected-text-tab', 'linky-selected-text-win');

	var elementTagsAll;
	if (p == 1)
		elementTagsAll = new Array('linky-all-sep-go', 'linky-all-tab-go', 'linky-all-win-go');
	else if (p == 2)
		elementTagsAll = new Array('linky-all-sep-mail', 'linky-all-tab-mail', 'linky-all-win-mail');
	else
		elementTagsAll = new Array('linky-all-sep', 'linky-all-tab', 'linky-all-win');

	var elementTagsPictureAll;
	if (p == 1)
		elementTagsPictureAll = new Array('linky-pictures-sep-go', 'linky-pictures-tab-go', 'linky-pictures-win-go');
	else if (p == 2)
		elementTagsPictureAll = new Array('linky-pictures-sep-mail', 'linky-pictures-tab-mail', 'linky-pictures-win-mail');
	else
		elementTagsPictureAll = new Array('linky-pictures-sep', 'linky-pictures-tab', 'linky-pictures-win');

	var elementTagsPictureLinks;
	if (p == 1)
		elementTagsPictureLinks = new Array('linky-pictures-links-page-sep-go', 'linky-pictures-links-page-tab-go', 'linky-pictures-links-page-win-go');
	else if (p == 2)
		elementTagsPictureLinks = new Array('linky-pictures-links-page-sep-mail', 'linky-pictures-links-page-tab-mail', 'linky-pictures-links-page-win-mail');
	else
		elementTagsPictureLinks = new Array('linky-pictures-links-page-sep', 'linky-pictures-links-page-tab', 'linky-pictures-links-page-win');

	var elementTagsPicturePage;
	if (p == 1)
		elementTagsPicturePage = new Array('linky-pictures-page-sep-go', 'linky-pictures-page-tab-go', 'linky-pictures-page-win-go');
	else if (p == 2)
		elementTagsPicturePage = new Array('linky-pictures-page-sep-mail', 'linky-pictures-page-tab-mail', 'linky-pictures-page-win-mail');
	else
		elementTagsPicturePage = new Array('linky-pictures-page-sep', 'linky-pictures-page-tab', 'linky-pictures-page-win');

	var elementTagsValidate;
	if (p == 1)
		elementTagsValidate = new Array('linky-validate-sep-go', 'linky-validate-go');
	else if (p == 2)
		elementTagsValidate = new Array('linky-validate-sep-mail', 'linky-validate-mail');
	else
		elementTagsValidate = new Array('linky-validate-sep', 'linky-validate');

	var elementTagsValidateLinks;
	if (p == 1)
		elementTagsValidateLinks = new Array('linky-validate-tab-go', 'linky-validate-win-go', 'linky-download-all-go');
	else if (p == 2)
		elementTagsValidateLinks = new Array('linky-validate-tab-mail', 'linky-validate-win-mail', 'linky-download-all-mail');
	else
		elementTagsValidateLinks = new Array('linky-validate-tab', 'linky-validate-win', 'linky-download-all');

	var elementTagsDownload;
	if (p == 1)
		elementTagsDownload = new Array('linky-download-sep-go', 'linky-download-go');
	else if (p == 2)
		elementTagsDownload = new Array('linky-download-sep-mail', 'linky-download-mail');
	else
		elementTagsDownload = new Array('linky-download-sep', 'linky-download');

	var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	var pref = prefService.getBranch(null);

	// try read the preferences. They might not be initialized yet
	try {
		this.pref_showselect = pref.getIntPref("extensions.linky.showselect");
	} catch(err) {
		linkyShared.trace("Unable to read the pref extensions.linky.showselect", 0, err);
	}
	try {
		this.pref_showopen = pref.getIntPref("extensions.linky.showopen");
	} catch(err) {
		linkyShared.trace("Unable to read the pref extensions.linky.showopen", 0, err);
	}
	try {
		this.pref_imgexts = pref.getCharPref("extensions.linky.imgexts");
	} catch(err) {
		linkyShared.trace("Unable to read the pref extensions.linky.imgexts", 0, err);
	}
	try {
		this.pref_validateURL = pref.getCharPref("extensions.linky.validateURL");
	} catch(err) {
		linkyShared.trace("Unable to read the pref extensions.linky.validateURL", 0, err);
	}
	try {
		this.pref_open_mailnews = pref.getBoolPref("extensions.linky.open_mailnews");
	} catch(err) {
		linkyShared.trace("Unable to read the pref extensions.linky.open_mailnews", 0, err);
	}
	try {
		this.pref_download_overwrite = pref.getIntPref("extensions.linky.download.overwrite");
	} catch(err) {
		linkyShared.trace("Unable to read the pref extensions.linky.download.overwrite", 0, err);
	}

	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.selectedlinks"), elementTagsSelected)) {
			elementTagsSelected = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.selectedlinks", 0, err);
	}

	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.selectedtextlinks"), elementTagsSelectedLink)) {
			elementTagsSelectedLink = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.selectedtextlinks", 0, err);
	}
	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.alllinks"), elementTagsAll)) {
			elementTagsAll = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.alllinks", 0, err);
	}
	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.piclinks"), elementTagsPictureAll)) {
			elementTagsPictureAll = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.piclinks", 0, err);
	}
	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.piclinksshow"), elementTagsPictureLinks)) {
			elementTagsPictureLinks = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.piclinksshow", 0, err);
	}
	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.picshow"), elementTagsPicturePage)) {
			elementTagsPicturePage = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.picshow", 0, err);
	}
	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.validatelinks"), elementTagsValidate)) {
			elementTagsValidate = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.validatelinks", 0, err);
		linkyContext.contextState(0, elementTagsValidate);
	}
	try {
		if (!linkyContext.contextState(pref.getBoolPref("extensions.linky.context.downloadlinks"), elementTagsDownload)) {
			elementTagsDownload = false;
		}
	} catch(err) {
		linkyShared.trace("Unable to use the pref extensions.linky.context.downloadlinks", 0, err);
		linkyContext.contextState(0, elementTagsDownload);
	}

	var disabled = true;
	var sel = linkyContext.getSelectionString();

	// check for selection
	if (elementTagsSelected) {
		if (sel.length > 0) {
			var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");
			if (lnks && lnks.length) {
				disabled = false;
			}
		}
		for (var i=0; i<elementTagsSelected.length; i++) {
			document.getElementById(elementTagsSelected[i]).setAttribute("disabled", disabled);
		}
	}

	// check for links in selection
	if (elementTagsSelectedLink) {
		disabled = true;
		if (sel.length > 0) {
			var selection = linkyContext.getSelectionObject();
			var urlPattern = /(\w*:\/\/)?([^\s\/]+\.(ac|ad|aero|ae|af|ag|ai|al|am|an|ao|aq|arpa|ar|as|at|au|aw|az|ba|bb|bd|be|bf|bg|bh|biz|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|com|coop|co|cr|cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|edu|ee|eg|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gov|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|info|int|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|mg|mh|mil|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|museum|mu|mv|mw|mx|my|mz|name|na|nc|net|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|org|pa|pe|pf|pg|ph|pk|pl|pm|pn|pro|pr|ps|pt|pw|py|qa|re|ro|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|sy|sz|tc|td|tf|tg|th|tj|tk|tm|tn|to|tp|tr|tt|tv|tw|tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|yu|za|zm|zw))((\/)(\S+)?|(\/)? |(\/)?$)/gmi;
			// RegExp parts:
			// 0 = complete result, may contain trailing space <-- Better not to evaluate this!
			// 1 = protocol, may be empty
			// 2 = domain
			// 3 = top level domain
			// 4 = complete path, may contain trailing space, or consist of a single space only, or be empty <-- Better not to evaluate this!
			// 5 = path root slash, may be empty
			// 6 = relative path, may be empty
			var found = urlPattern.exec(selection);
			if (found) {
				disabled = false;
			}
		}
		for (i=0; i<elementTagsSelectedLink.length; i++) {
			document.getElementById(elementTagsSelectedLink[i]).setAttribute("disabled",disabled);
		}
	}

	// check for links
	if (elementTagsValidate) {
		disabled = false;
		lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");
		if (lnks && !lnks.length) {
			disabled = true;
		}
		for (i=0; i<elementTagsValidateLinks.length; i++) {
			document.getElementById(elementTagsValidateLinks[i]).setAttribute("disabled",disabled);
		}
	}

	// check for links
	if (elementTagsAll) {
		disabled = false;
		if ("links" in document.commandDispatcher.focusedWindow.document && document.commandDispatcher.focusedWindow.document.links.length < 1) {
		linkyShared.trace("dis: true");
			disabled = true;
		}
		for (i=0; i<elementTagsAll.length; i++) {
			document.getElementById(elementTagsAll[i]).setAttribute("disabled",disabled);
		}
	}

	// check for picture links
	if (elementTagsPictureAll || elementTagsPictureLinks) {
		disabled = false;
		lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");
		if (lnks && lnks.length) {
			// update the info item with current numbers of links and images
			var msg = document.getElementById("bundle_linky").getString("linky-info");
			msg = linkyShared.replaceInsert(msg, 1, lnks.length);
			var imgs = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "img");
			if (imgs && imgs.length) {
				msg = linkyShared.replaceInsert(msg, 2, imgs.length);
			} else {
				msg = linkyShared.replaceInsert(msg, 1, 0);
			}
			if (p == 1)
				document.getElementById("linky-info-go").label = msg;
			else if (p == 2)
				document.getElementById("linky-info-mail").label = msg;
			else
				document.getElementById("linky-info").label = msg;

			// because of performance only check the first 100 links
			if (lnks.length > 100) {
				disabled = false;
				linkyShared.trace("More than 100 links. There's " + lnks.length);
			} else {
				var badurl=0;
				for (i = 0; i < lnks.length; i++) {
					if (linkyContext.isLinkImage(lnks[i].href)) {
						badurl=1;
						break;
					}
				}
				if (badurl) {
					disabled = false;
				} else {
					disabled = true;
				}
			}
		} else {
			if (p == 1)
				document.getElementById("linky-info-go").label=document.getElementById("bundle_linky").getString("linky-info-na");
			else if (p == 2)
				document.getElementById("linky-info-mail").label=document.getElementById("bundle_linky").getString("linky-info-na");
			else
				document.getElementById("linky-info").label=document.getElementById("bundle_linky").getString("linky-info-na");
		}
		for (i=0; i<elementTagsPictureAll.length; i++) {
			document.getElementById(elementTagsPictureAll[i]).setAttribute("disabled",disabled);
		}
		for (i=0; i<elementTagsPictureLinks.length; i++) {
			document.getElementById(elementTagsPictureLinks[i]).setAttribute("disabled",disabled);
		}
	}

	// check for images on page
	if (elementTagsPicturePage) {
		disabled = false;
		imgs = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "img");
		if (imgs && !imgs.length) {
			disabled = true;
		}
		for (i=0; i<elementTagsPicturePage.length; i++) {
			document.getElementById(elementTagsPicturePage[i]).setAttribute("disabled",disabled);
		}
	}
}

// Show the Preferences dialog
LinkyContext.prototype.doPref = function() {
	linkyShared.trace("doPref");
	// have to use try catch since Mozilla Firebird dont support goPreferences
	try {
		goPreferences('linky-pref-panel', 'chrome://linky/content/pref-panel.xul', 'linky-pref-panel');
	} catch(err) {
		openDialog("chrome://linky/content/pref-panel-extension.xul", "PrefWindow", "chrome,titlebar,resizable,modal");
	}
}

// open selected links
// opentype: 1 = in new tab, 2 = in new window
LinkyContext.prototype.doSelected = function(opentype, event) {
	linkyShared.trace("doSelected");
	if (linkyContext.getSelectionString().length < 1) {
		linkyShared.trace("Selection is empty!", 1);
	} else {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = false;
		if (event.shiftKey) {
			linky_forceselect = true;
		}
		if ("links" in document.commandDispatcher.focusedWindow.document) {
			var lnks = document.commandDispatcher.focusedWindow.document.links;
			if (lnks.length < 1) {
				linkyShared.trace("No links in selection!", 1);
			} else {
				var selection = linkyContext.getSelectionObject();
				var urlhash = new Object();
				var urls = new Array();
				var len = lnks.length;
				for (var i = 0; i < len; ++i) {
					var u = lnks[i].href
					if (selection.containsNode(lnks[i], true) && linkyContext.isLinkOpenable(lnks[i].href)) {
						urlhash[lnks[i].href] = true;
					}
				}
				for (i in urlhash) {
					urls[urls.length] = i;
				}
				if (urls.length > 0) {
					if (linky_forceselect || urls.length >= this.pref_showselect) {
						window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
					} else {
						for (i = 0; i < urls.length; i++) {
							linkyContext.openLink(urls[i], opentype);
						}
					}
				} else {
					linkyShared.trace("No links in selection!", 1);
				}
			}
		} else {
			linkyShared.trace("Cant access links in document!", 1);
		}
	}
}

// open selected text links
// opentype: 1 = in new tab, 2 = in new window
LinkyContext.prototype.doSelectedText = function(opentype, event) {
	linkyShared.trace("doSelectedText");
	if (linkyContext.getSelectionString().length < 1) {
		linkyShared.trace("Selection is empty!", 1);
	} else {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = false;
		if (event.shiftKey) {
			linky_forceselect = true;
		}
		var selection = linkyContext.getSelectionObject();
		var urls = new Array();
		var urlhash = new Object();
		var urlPattern = /(\w*:\/\/)?([^\s\/]+\.(ac|ad|aero|ae|af|ag|ai|al|am|an|ao|aq|arpa|ar|as|at|au|aw|az|ba|bb|bd|be|bf|bg|bh|biz|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|com|coop|co|cr|cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|edu|ee|eg|er|es|et|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gov|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|info|int|in|io|iq|ir|is|it|je|jm|jo|jp|ke|kg|kh|ki|km|kn|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|mg|mh|mil|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|museum|mu|mv|mw|mx|my|mz|name|na|nc|net|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|org|pa|pe|pf|pg|ph|pk|pl|pm|pn|pro|pr|ps|pt|pw|py|qa|re|ro|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|sy|sz|tc|td|tf|tg|th|tj|tk|tm|tn|to|tp|tr|tt|tv|tw|tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|ye|yt|yu|za|zm|zw))((\/)(\S+)?|(\/)? |(\/)?$)/gmi;
		// RegExp parts:
		// 0 = complete result, may contain trailing space <-- Better not to evaluate this!
		// 1 = protocol, may be empty
		// 2 = domain
		// 3 = top level domain
		// 4 = complete path, may contain trailing space, or consist of a single space only, or be empty <-- Better not to evaluate this!
		// 5 = path root slash, may be empty
		// 6 = relative path, may be empty
		var found = urlPattern.exec(selection);
		while (found) {
			if (!found[1]) {
				found[1] = "http://";
			}
			if (!found[5]) {
				found[5] = "/";
			}
			if (!found[6]) {
				found[6] = "";
			}
			var url = found[1] + found[2] + found[5] + found[6];
			if (linkyContext.isLinkOpenable(url)) {
				urlhash[url] = true;
			}
			// proceed to the next link
			found = urlPattern.exec(selection);
		}
		for (var i in urlhash) {
			urls[urls.length] = i;
		}
		if (urls.length > 0) {
			if (linky_forceselect || urls.length >= this.pref_showselect) {
				window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
			} else {
				for (var i = 0; i < urls.length; i++) {
					linkyContext.openLink(urls[i], opentype);
				}
			}
		} else {
			linkyShared.trace("No links in selection!", 1);
		}
	}
}

// Show all images
// opentype: 3 = in one new tab, 4 = in one new window
LinkyContext.prototype.doImages = function(opentype, event) {
	linkyShared.trace("doImages");
	if ("images" in document.commandDispatcher.focusedWindow.document) {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = 0;
		if (event.shiftKey) {
			linky_forceselect = 1;
		}
		var imgs = document.commandDispatcher.focusedWindow.document.images;

		var urlhash = new Object();
		var urls = new Array();
		var len = imgs.length;
		for (var i = 0; i < len; ++i) {
			var u = imgs[i].src;
			if (u) {
				urlhash[u] = true;
			}
		}
		for (i in urlhash) {
			urls[urls.length] = i;
		}

		if (urls.length > 0) {
			if (linky_forceselect || urls.length >= this.pref_showselect) {
				window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
			} else {
				var doc;
				if (opentype == 3) {
					try {
						doc = linkyContext.generateDocument("", true);
					} catch(err) {
						linkyShared.trace("Unable to add new tab", 1, err);
					}
				} else if (opentype == 4) {
					try {
						doc = linkyContext.generateDocument("", false);
					} catch(err) {
						linkyShared.trace("Unable to add new window", 1, err);
					}
				} else {
					linkyShared.trace("Unknown opentype: " + opentype, 1);
				}
				for (i = 0; i < urls.length; i++) {
					linkyContext.openLink(urls[i], 3, doc);
				}
			}
		} else {
			linkyShared.trace("No images detected!", 1);
		}
	} else {
		linkyShared.trace("Cant access images in document!", 1);
	}
}

// Open all links
// opentype: 1 = in new tab, 2 = in new window
LinkyContext.prototype.doAll = function(opentype, event) {
	linkyShared.trace("doAll");
	var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");

	if (lnks && lnks.length) {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = 0;
		if (event.shiftKey) {
			linky_forceselect = 1;
		}

		var urlhash = new Object();
		var urls = new Array();
		var len = lnks.length;
		for (var i = 0; i < len; ++i) {
			var u = lnks[i].href
			if (linkyContext.isLinkOpenable(u)) {
				urlhash[u] = true;
			}
		}
		for (i in urlhash) {
			urls[urls.length] = i;
		}

		if (urls.length > 0) {
			if (linky_forceselect || urls.length >= this.pref_showselect) {
				window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
			} else {
				for (i = 0; i < urls.length; i++) {
					linkyContext.openLink(urls[i], opentype);
				}
			}
		} else {
			linkyShared.trace("No valid links found!", 1);
		}
	} else {
		linkyShared.trace("No valid links found!", 1);
	}
}

// Show all image links on one page
// opentype: 1 = in new tab, 2 = in new window, 3 = in one new tab, 4 = in one new window
LinkyContext.prototype.doAllPics = function(opentype, event) {
	linkyShared.trace("doAllPics");
	var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");

	if (lnks && lnks.length) {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = 0;
		if (event.shiftKey) {
			linky_forceselect = 1;
		}

		var urlhash = new Object();
		var urls = new Array();
		var len = lnks.length;
		for (var i = 0; i < len; ++i) {
			var u = lnks[i].href
			if (linkyContext.isLinkImage(u)) {
				urlhash[u] = true;
			}
		}
		for (i in urlhash) {
			urls[urls.length] = i;
		}

		if (urls.length > 0) {
			if (linky_forceselect || urls.length >= this.pref_showselect) {
				window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
			} else {
				if (opentype < 3) {
					for (i = 0; i < urls.length; i++) {
						linkyContext.openLink(urls[i], opentype);
					}
				} else {
					var doc;
					if (opentype == 3) {
						try {
							doc = linkyContext.generateDocument("", true);
						} catch(err) {
							linkyShared.trace("Unable to add new tab", 1, err);
						}
					} else if (opentype == 4) {
						try {
							doc = linkyContext.generateDocument("", true);
						} catch(err) {
							linkyShared.trace("Unable to add new window", 1, err);
						}
					} else {
						linkyShared.trace("Unknown opentype: " + opentype);
					}
					for (i = 0; i < urls.length; i++) {
						linkyContext.openLink(urls[i], 3, doc);
					}
				}
			}
		} else {
			linkyShared.trace("No image links found!", 1);
		}
	} else {
		linkyShared.trace("Cant access links in document!", 1);
	}
}

// validate all links
// opentype: 1 = in new tab, 2 = in new window
LinkyContext.prototype.doValidateAll = function(opentype, event) {
	linkyShared.trace("doValidateAll");
	var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");

	if (lnks && lnks.length) {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = 0;
		if (event.shiftKey) {
			linky_forceselect = 1;
		}

		var urlhash = new Object();
		var urls = new Array();
		var len = lnks.length;
		for (var i = 0; i < len; ++i) {
			var u = lnks[i].href
			if (linkyContext.isLinkOpenable(u, 1)) {
				urlhash[this.pref_validateURL + escape(u)] = true;
			}
		}
		for (i in urlhash) {
			urls[urls.length] = i;
		}

		if (urls.length > 0) {
			if (linky_forceselect || urls.length >= this.pref_showselect) {
				window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
			} else {
				for (i = 0; i < urls.length; i++) {
					linkyContext.openLink(urls[i], opentype);
				}
			}
		} else {
			linkyShared.trace("No valid links found!", 1);
		}
	} else {
		linkyShared.trace("Cant access links in document!", 1);
	}
}

// validate selected links
// opentype: 1 = in new tab, 2 = in new window
LinkyContext.prototype.doValidateSelected = function(opentype, event) {
	linkyShared.trace("doValidateSelected");
	if (linkyContext.getSelectionString().length < 1) {
		linkyShared.trace("Selection is empty!", 1);
	} else {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = false;
		if (event.shiftKey) {
			linky_forceselect = true;
		}
		var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");
		if (lnks && lnks.length) {
			var selection = linkyContext.getSelectionObject();
			var urlhash = new Object();
			var urls = new Array();
			var len = lnks.length;
			for (var i = 0; i < len; ++i) {
				var u = lnks[i].href
				if (selection.containsNode(lnks[i], true) && linkyContext.isLinkOpenable(u, 1)) {
					urlhash[this.pref_validateURL + escape(u)] = true;
				}
			}
			for (i in urlhash) {
				urls[urls.length] = i;
			}

			if (urls.length > 0) {
				if (linky_forceselect || urls.length >= this.pref_showselect) {
					window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, opentype);
				} else {
					for (i = 0; i < urls.length; i++) {
						linkyContext.openLink(urls[i], opentype);
					}
				}
			} else {
				linkyShared.trace("No links in selection!", 1);
			}
		} else {
			linkyShared.trace("Cant access links in document!", 1);
		}
	}
}
// download all links
LinkyContext.prototype.doDownloadAll = function(event) {
	linkyShared.trace("doDownloadAll");
	var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");

	if (lnks && lnks.length) {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = 0;
		if (event.shiftKey) {
			linky_forceselect = 1;
		}

		var urlhash = new Object();
		var urls = new Array();
		var len = lnks.length;
		for (var i = 0; i < len; ++i) {
			var u = lnks[i].href
			if (linkyContext.isLinkSaveable(u)) {
				urlhash[u] = true;
			}
		}
		for (i in urlhash) {
			urls[urls.length] = i;
		}

		if (urls.length > 0) {
			if (linky_forceselect || urls.length >= this.pref_showselect) {
				window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, -1);
			} else {
				var dir = linkyContext.getDirectory();
				if (dir) {
					var ref = linkyContext.getReferer(document.commandDispatcher.focusedWindow.document);
					for (i = 0; i < urls.length; i++) {
						linkyContext.downloadLink(urls[i], ref, dir);
					}
				} else {
					linkyShared.trace("User cancelled out", 0);
				}
			}
		} else {
			linkyShared.trace("No downloadable links found!", 1);
		}
	} else {
		linkyShared.trace("Cant access links in document!", 1);
	}
}

// download selected links
LinkyContext.prototype.doDownloadSelected = function(event) {
	linkyShared.trace("doDownloadSelected");
	if (linkyContext.getSelectionString().length < 1) {
		linkyShared.trace("Selection is empty!", 1);
	} else {
		// holding down the SHIFT key while pressing the menuitem forces the select links dialog
		var linky_forceselect = false;
		if (event.shiftKey) {
			linky_forceselect = true;
		}
		var lnks = document.commandDispatcher.focusedWindow.document.getElementsByTagNameNS("*", "a");
		if (lnks && lnks.length) {
			var selection = linkyContext.getSelectionObject();
			var urlhash = new Object();
			var urls = new Array();
			var len = lnks.length;
			for (var i = 0; i < len; ++i) {
				var u = lnks[i].href
				if (selection.containsNode(lnks[i], true) && linkyContext.isLinkSaveable(u)) {
					urlhash[u] = true;
				}
			}
			for (i in urlhash) {
				urls[urls.length] = i;
			}
			if (urls.length > 0) {
				if (linky_forceselect || urls.length >= this.pref_showselect) {
					window.openDialog("chrome://linky/content/select-links.xul","_blank","chrome,modal,centerscreen,resizable=yes", urls, -1);
				} else {
					var dir = linkyContext.getDirectory();
					if (dir) {
						var ref = linkyContext.getReferer(document.commandDispatcher.focusedWindow.document);
						for (i = 0; i < urls.length; i++) {
							linkyContext.downloadLink(urls[i], ref, dir);
						}
					} else {
						linkyShared.trace("User cancelled out", 0);
					}
				}
			} else {
				linkyShared.trace("No links in selection!", 1);
			}
		} else {
			linkyShared.trace("Cant access links in document!", 1);
		}
	}
}

//
LinkyContext.prototype.downloadLink = function(u, ref, dir) {
	linkyShared.trace("downloadLink");
	var data = {url: u, fileName: null, filePickerTitle: null, document: null, bypassCache: false, referer: ref, directory: dir};
	var sniffer = new nsHeaderSniffer(u, linkyContext.myfoundHeaderInfo, data);
}

//
LinkyContext.prototype.myfoundHeaderInfo = function(aSniffer, aData) {
	linkyShared.trace("myfoundHeaderInfo");
	// get the default local filename
	var defaultFilename = getDefaultFileName(aData.fileName, aSniffer.suggestedFileName, aSniffer.uri, null);

	// create the local file
	var lf = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	var defaultDir;
	if (aData.directory) {
		defaultDir = aData.directory;
	} else {
		linkyShared.trace("No directory specified", 0);
		return;
	}

	lf.initWithPath(defaultDir);
	lf.append(defaultFilename);
	linkyShared.trace("full path: " + lf.path, 0);
	var lftmp = lf.path;

 	if (lf.exists()) {
 		switch(linkyContext.pref_download_overwrite) {
	 		// overwrite
	 		case 0: break;

	 		// prompt
	 		case 1: if (!confirm(lf.path + ' exists! Do you want to overwrite?')) return;
	 			break;

	 		// ignore
	 		case 2: return;

	 		// auto-rename
	 		case 3: {
	 			var i = 0;
	 			do {
	 				lf.initWithPath(defaultDir);
	 				lf.append(i++ + "_" + defaultFilename);
	 				// to avoid endless recurssion
	 				if (i > 500) {
	 					linkyShared.trace("Recurssion detected! Unable to save URL.", 0);
	 					return;
	 				}
	 			} while (lf.exists());
	 			break;
	 		}

	 		// should never happen!
	 		default: {
	 			linkyShared.trace("Error! pref_download_overwrite was " + linkyContext.pref_download_overwrite, 0);
	 			return;
	 		}
 		}
 	}
	if (lf.path != lftmp)
		linkyShared.trace("new full path: " + lf.path, 0);

	// setup download data
	var source = aSniffer.uri;
	var persistArgs = {source : source, contentType : aSniffer.contentEncodingType, target : lf, postData : null, bypassCache : false};

	// create a persistant browser
	var persist = makeWebBrowserPersist();
	const nsIWBP = Components.interfaces.nsIWebBrowserPersist;
	const flags = nsIWBP.PERSIST_FLAGS_NO_CONVERSION | nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES | nsIWBP.PERSIST_FLAGS_FROM_CACHE;

	linkyShared.trace("about to save " + source.spec + " into " + lf.path, 0)

	// do the actual download
	var dl;
	try {
		dl = Components.classes["@mozilla.org/transfer;1"].createInstance(Components.interfaces.nsITransfer);
	} catch(err) {
		dl = Components.classes["@mozilla.org/download;1"].createInstance(Components.interfaces.nsIDownload);
	}

	try {
		//dl.init(source, persistArgs.target, null, null, null, persist);
		// hack hack: version number should be dynamic
		persist.saveURI(source, null, aData.referer, persistArgs.postData, "Linky: 2.1.0\r\n", persistArgs.target);
	} catch(err) {
		linkyShared.trace("ERROR! Unable to download: " + source.spec, 0, err);
	}
}

// create the object
const linkyContext = new LinkyContext();
