/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Henrik Gemal.
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Henrik Gemal http://gemal.dk
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var linky_version;

// What prefs are we handling?
var checkboxes  = ["extensions.linky.context.selectedlinks", "extensions.linky.context.selectedtextlinks", "extensions.linky.context.alllinks", "extensions.linky.context.piclinks", "extensions.linky.context.piclinksshow", "extensions.linky.context.validatelinks", "extensions.linky.context.downloadlinks", "extensions.linky.context.picshow"];
var radiogroups = ["extensions.linky.showopen"];
var selectboxes = ["extensions.linky.showselect"];

// This gets called automatically from Mozilla Application Suite
function Startup() {
	// set default prefs
	linkyPrefSet();

	// set the text to include the version number
	if (document.getElementById("bundle_linky")) {
		try {
			var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
			var addon = em.getItemForID("linky@gemal.dk");
			linky_version = addon.version;
		} catch(e) {
			if (InstallTrigger && InstallTrigger.getVersion) {
				// hack hack: somehow this stopped working! see https://bugzilla.mozilla.org/show_bug.cgi?id=208016
				linky_version = InstallTrigger.getVersion("linky");
			}
		}
		if (linky_version != null) {
			var vermsg = document.getElementById("bundle_linky").getString("linky-pref-version");
			vermsg = linkyShared.replaceInsert(vermsg, 1, linky_version);
			document.getElementById("linky.version").setAttribute("value", vermsg);
		}
	}
}

function linkyPrefSet() {
	var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	// if extensions.linky.showopen pref isn't there set all prefs
	try {
		var tmp = pref.getIntPref("extensions.linky.showopen");
	} catch(e) {
		linkyShared.trace("Setting default prefs");
		pref.setIntPref("extensions.linky.showopen", 2);
		pref.setIntPref("extensions.linky.showselect", 0);
		pref.setBoolPref("extensions.linky.context.alllinks", true);
		pref.setBoolPref("extensions.linky.context.piclinks", true);
		pref.setBoolPref("extensions.linky.context.piclinksshow", true);
		pref.setBoolPref("extensions.linky.context.picshow", true);
		pref.setBoolPref("extensions.linky.context.selectedlinks", true);
		pref.setBoolPref("extensions.linky.context.selectedtextlinks", true);
		pref.setBoolPref("extensions.linky.context.validatelinks", false);
		pref.setBoolPref("extensions.linky.context.downloadlinks", false);
		pref.setIntPref("extensions.linky.download.overwrite", 3);
		pref.setBoolPref("extensions.linky.open_mailnews", false);
		pref.setCharPref("extensions.linky.imgexts", "jpg|jpeg|gif|png|bmp");
		pref.setCharPref("extensions.linky.validateURL", "http://validator.w3.org/check?verbose=1&ss=1&uri=");
	}
}

// This has to be called manually from Mozilla Firebird
function linkyPrefInit() {
	// set default prefs
	linkyPrefSet();

	// Init the prefs
	var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	for (var i = 0; i < checkboxes.length; ++i) {
		var checkbox = document.getElementById(checkboxes[i]);
		try {
			checkbox.checked = pref.getBoolPref(checkbox.getAttribute("prefstring"));
		} catch(err) {
			linkyShared.trace("Unable to init pref " + checkboxes[i], 1, err);
			checkbox.checked = true;
		}
	}

	for (i = 0; i < radiogroups.length; ++i) {
		var radiogroup = document.getElementById(radiogroups[i]);
		try {
			radiogroup.selectedItem = radiogroup.getElementsByAttribute("value", pref.getIntPref(radiogroup.getAttribute("prefstring")))[0];
		} catch(err) {
			linkyShared.trace("Unable to init pref " + radiogroups[i], 1, err);
			// radiogroup.checked = true;
		}
	}

	for (i = 0; i < selectboxes.length; ++i) {
		var selectbox = document.getElementById(selectboxes[i]);
		try {
			selectbox.selectedItem = selectbox.getElementsByAttribute("value", pref.getIntPref(selectbox.getAttribute("prefstring")))[0];
		} catch(err) {
			linkyShared.trace("Unable to init pref " + selectboxes[i], 1, err);
			selectbox.selectIndex = 3;
		}
	}

	// Manually call the Startup since Mozilla Firebird dont call it
	Startup();
}

function linkyPrefSave() {
	// Manually save prefs since Mozilla Firebird doesn't do it
	var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

	for (var i = 0; i < checkboxes.length; ++i) {
		var checkbox = document.getElementById(checkboxes[i]);
		try {
			pref.setBoolPref(checkbox.getAttribute("prefstring"), checkbox.checked);
		} catch(err) {
			linkyShared.trace("Unable to save pref " + checkboxes[i], 1, err);
		}
	}
	for (i = 0; i < radiogroups.length; ++i) {
		var radiogroup = document.getElementById(radiogroups[i]);
		try {
			pref.setIntPref(radiogroup.getAttribute("prefstring"), radiogroup.selectedItem.value);
		} catch(err) {
			linkyShared.trace("Unable to save pref " + radiogroups[i], 1, err);
		}
	}
	for (i = 0; i < selectboxes.length; ++i) {
		var selectbox = document.getElementById(selectboxes[i]);
		try {
			pref.setIntPref(selectbox.getAttribute("prefstring"), selectbox.selectedItem.value);
		} catch(err) {
			linkyShared.trace("Unable to save pref " + selectboxes[i], 1, err);
		}
	}
}

function linkyUpdate() {
//	document.commandDispatcher.focusedWindow.open("http://gemal.dk/mozilla/linky.html?ver=" + linky_version);
	document.commandDispatcher.focusedWindow.open("http://www.smart-roadster-club.de/off-topic/mozilla/extensions/linky/");
}
