/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Henrik Gemal.
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Henrik Gemal http://gemal.dk
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Put everything into a namespace so that we dont pollute
var LinkySelect = function() {
}

// select all already visisted links
LinkySelect.prototype.checkVisited = function() {
	var gGlobalHistory = opener.Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIGlobalHistory);
	var cbox = document.getElementById("cvbox").checked;
	if (!cbox) {
		cbox = false;
	}
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		if (gGlobalHistory.isVisited(tmp.getAttribute("value"))) {
			tmp.setAttribute("checked", cbox);
		}
	}
}

// mark selected links visisted
LinkySelect.prototype.markVisited = function() {
	var gGlobalHistory = opener.Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIGlobalHistory);

	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	var j = 0;
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		if (tmp.getAttribute("checked") == "true") {
			gGlobalHistory.addPage(tmp.getAttribute("value"));
			j++;
		}
	}
	if (j) {
		alert(j + " links marked as visited.");
	}
}

// select all links
LinkySelect.prototype.checkAll = function() {
	var cbox = document.getElementById("cbox").checked;
	if (!cbox) {
		cbox = false;
	}
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		tmp.setAttribute("checked", cbox);
	}
}

// Copy links to the clipboard
// t indicates the type
//  0 : only the selected links are copied
//  1 : all links are copied
LinkySelect.prototype.copyClipboard = function(t) {
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	var txt = "";
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		var s = "";
		if (t) {
			s = tmp.getAttribute("value");
		} else {
			if (tmp.getAttribute("checked") == "true") {
				s = tmp.getAttribute("value");
			}
		}
		if (s) {
			txt += s;
			if (i < last-1) {
				txt += "\n";
			}
		}
	}
	if (txt) {
		const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
		gClipboardHelper.copyString(txt);
	}
}

// Bookmark selected links as group
LinkySelect.prototype.bookmarkLinks = function() {
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	var groups = new Array();
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		if (tmp.getAttribute("checked") == "true") {
			groups[groups.length] = tmp.getAttribute("value");
		}
	}
	if (groups.length) {
		var gname = prompt(opener.document.getElementById("bundle_linky").getString("linky-select-bookmarkgroupname"));
		if (gname) {
			const gBookmark = Components.classes["@mozilla.org/browser/bookmarks-service;1"].getService(Components.interfaces.nsIBookmarksService);
			const group = gBookmark.createGroupInContainer("linky", rFolder, -1);
			var url;
			for (i = 0; i < groups.length; ++i) {
				url = getNormalizedURL(groups[i]);
				gBookmark.createBookmarkInContainer(url, url, null, null, null, group, -1);
			}
		}
	}
}

// Sort the links
// t 1 : sort by link
// t 2 : not sorted
// t 3 : sort by hostname
LinkySelect.prototype.sortSelect = function(t) {
	// what sort order to use
	var slnks;
	if (t == 2) {
		// not sorted but by document.links object
		slnks = lnks_org;
	} else if (t == 3) {
		// sorted by hostname

		var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
		// use own sort function
		var compareFunc = function compare(first, second) {
			uri.spec = first;
			try {
				var fu = uri.host;
			} catch(err) {
				opener.linkyShared.trace("Unable to get host from " + first,1,err)
			}
			uri.spec = second;
			try {
				var su = uri.host;
			} catch(err) {
				opener.linkyShared.trace("Unable to get host from " + second,1,err)
			}

			if (fu < su)
				return -1;
			else if (fu > su)
				return 1;
			else
				if (first < second)
					return -1;
				else if (first > second)
					return 1;
				else
					return 0;
		}
		slnks = lnks.sort(compareFunc);
	} else {
		// sort by URL
		slnks = lnks.sort();
	}

	// remove the links
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	for (var i = last - 1; i > -1; i--) {
		selectlinks.removeItemAt(i);
	}

	// add the sorted links
	for(i = 0; i < slnks.length; ++i) {
		var itemNode = document.createElement("listitem");
		itemNode.setAttribute("type", "checkbox");
		itemNode.setAttribute("value", slnks[i]);
		itemNode.setAttribute("label", slnks[i]);
		itemNode.setAttribute("checked", true);
		selectlinks.appendChild(itemNode);
	}
}

// select links based on the hostname
LinkySelect.prototype.checkHost = function(h) {
	var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
	// select the links
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		uri.spec = tmp.getAttribute("value");
		try {
			var tmphost = uri.host;
		} catch(err) {
			opener.linkyShared.trace("Unable to get host from " + uri.spec,1,err)
		}
		if (h == tmphost) {
			tmp.setAttribute("checked", true);
		} else {
			tmp.setAttribute("checked", false);
		}
	}
}

// select/deselect links based on text input
LinkySelect.prototype.checkPartial = function(pat, f) {
	if (!pat || pat == "")
		return;

	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		if (tmp.getAttribute("value").indexOf(pat) != -1) {
			tmp.setAttribute("checked", (f ? true : false));
		} else {
			tmp.setAttribute("checked", (f ? false : true));
		}
	}
}

// remove part of selected links based on text input
LinkySelect.prototype.removePartial = function(pat) {
	if (!pat || pat == "")
		return;

	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	var re = new RegExp(pat, "g");
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		if (tmp.getAttribute("checked") == "true") {
			var str = tmp.getAttribute("value");
			var strnew = str.replace(re, "");
			if (str != strnew) {
				tmp.setAttribute("label", strnew);
				tmp.setAttribute("value", strnew);
			}
		}
	}
}

// unescaepe links
// http://gemal.dk/?url=http://gemal.dk?p=1 becomes http://gemal.dk?p=1
// http://gemal.dk/?url=http://gemal.dk?p=1&p=2 becomes http://gemal.dk?p=1
LinkySelect.prototype.unescLinks = function() {
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	var URL=/.*\W(\w+:\/\/.*?)(&|$)/i;
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		var val = tmp.getAttribute("value");
		var lnk = URL.exec(val);
		if (lnk && lnk[1]) {
			tmp.setAttribute("value", lnk[1]);
			tmp.setAttribute("label", lnk[1]);
		}
	}
}

// open the selected links
LinkySelect.prototype.openLinks = function() {
	var selectlinks = document.getElementById("selectlinks");
	var last = selectlinks.getRowCount();
	var selnum = 0;
	var urlhash = new Object();
	for (var i = 0; i < last; i++) {
		var tmp = selectlinks.getItemAtIndex(i);
		if (tmp.getAttribute("checked") == "true") {
			selnum++;
			urlhash[tmp.getAttribute("value")] = true;
		}
	}

	if (selnum) {
		if (selnum > 999 && (opentype == 1 || opentype == 3)) {
			opener.linkyShared.trace("Cant open more than 999 links. You tried to open " + selnum + " link. Please deselect some links!", 1);
			return;
		}
	
		// get referer and directory but only if we are doing download
		var ref;
		var dir;
		if (opentype == -1) {
			dir = document.getElementById("linky-select-download-dir").getAttribute("value");
			if (!dir) {
				opener.linkyShared.trace("Please specify a Download Folder!", 1);
				return;
			}
			ref = opener.linkyContext.getReferer(opener.document.commandDispatcher.focusedWindow.document);
		}

		var doc;
		for (i in urlhash) {
			if (opentype > 2) {
				if (!doc) {
					if (opentype == 3) {
						try {
							doc = opener.linkyContext.generateDocument("", true);
						} catch(err) {
							opener.linkyShared.trace("Unable to add new tab", 1, err);
						}
					} else if (opentype == 4) {
						try {
							doc = opener.linkyContext.generateDocument("", true);
						} catch(err) {
							opener.linkyShared.trace("Unable to add new window", 1, err);
						}
					} else {
						opener.linkyShared.trace("Error 1: Unknown opentype: " + opentype);
					}
				}
				opener.linkyContext.openLink(i, 3, doc);
			} else if (opentype == -1) {
				opener.linkyContext.downloadLink(i, ref, dir);
			} else {
				opener.linkyContext.openLink(i, opentype);
			}
		}
	}
	self.close();
}

// do the startup thing
LinkySelect.prototype.startup = function() {
	// if we're doing download change the text and add Download Folder
	if (opentype == -1) {
		document.getElementById("linky-select-intro").label = opener.document.getElementById("bundle_linky").getString("linky-select-intro");
		document.getElementById("linky-select-openlinks").label = opener.document.getElementById("bundle_linky").getString("linky-select-openlinks");
		document.getElementById("linky-select-openlinks").setAttribute("tooltiptext", opener.document.getElementById("bundle_linky").getString("linky-select-openlinks-tooltip"));
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefService.getBranch(null);
		var dir;
		try {
			dir = pref.getComplexValue("browser.download.dir", Components.interfaces.nsILocalFile);
		} catch(err) {
			opener.linkyShared.trace("Unable to get pref browser.download.dir", 0, err);
		}
		if (dir) {
			document.getElementById("linky-select-download-dir").setAttribute("value", dir.path);
		}
		document.getElementById("linky-select-download").setAttribute("hidden", false);
	}
	var selectlinks = document.getElementById("selectlinks");
	for(var i = 0; i < lnks.length; ++i) {
		var itemNode = document.createElement("listitem");
		itemNode.setAttribute("type", "checkbox");
		itemNode.setAttribute("value", lnks[i]);
		itemNode.setAttribute("label", lnks[i]);
		itemNode.setAttribute("checked", true);
		selectlinks.appendChild(itemNode);
	}
}

// do the startup thing
LinkySelect.prototype.doDir = function() {
	var dir = opener.linkyContext.getDirectory();
	if (dir) {
		document.getElementById("linky-select-download-dir").setAttribute("value", dir);
	}
}

const linkySelect = new LinkySelect;

var lnks;
var opentype;
if ('arguments' in window) {
	lnks = window.arguments[0];
	opentype = window.arguments[1];
} else {
	opener.linkyShared.trace("Error 1: No arguments provided!", 1);
}
// preserve the original lnks object
var lnks_org = new Array(lnks.length);
for (var i = 0; i < lnks_org.length; i++) {
	lnks_org[i] = lnks[i];
}

